# Error Handling

This document provides information on how CoinsSend API handles errors and how to properly handle them in your applications.

## Error Response Format

All API errors are returned with appropriate HTTP status codes and a consistent JSON response format:

```json
{
  "status": "error",
  "error": {
    "code": 400,
    "message": "Error description"
  }
}
```

The `code` field corresponds to the HTTP status code and the `message` field provides a human-readable description of the error.

## Common HTTP Status Codes

| Status Code | Name | Description |
|-------------|------|-------------|
| 400 | Bad Request | The request was malformed or contained invalid parameters |
| 401 | Unauthorized | Authentication failed (invalid or missing Merchant/Sign headers) |
| 403 | Forbidden | The API key doesn't have permission for the requested operation |
| 404 | Not Found | The requested resource doesn't exist |
| 422 | Unprocessable Entity | Validation errors in the request data |
| 429 | Too Many Requests | Rate limit exceeded |
| 500 | Internal Server Error | An unexpected error occurred on the server |

## Validation Errors

For validation errors (HTTP 422), the response includes details about the validation failures:

```json
{
  "status": "error",
  "error": {
    "code": 422,
    "message": "Validation failed",
    "errors": {
      "amount": ["The amount field is required."],
      "order_id": ["The order id field must be unique."]
    }
  }
}
```

## Common Error Codes by Endpoint

### Authentication Errors

| HTTP Code | Error Code | Description |
|-----------|------------|-------------|
| 400 | MISSING_HEADERS | Merchant or Sign header is missing |
| 401 | INVALID_MERCHANT | The provided merchant ID is invalid |
| 401 | INVALID_SIGN | The provided signature is invalid |

### Invoice Errors

| HTTP Code | Error Code | Description |
|-----------|------------|-------------|
| 400 | INVALID_REQUEST | An invoice with the same order_id already exists for this merchant |
| 422 | VALIDATION_ERROR | The amount parameter is missing |
| 422 | VALIDATION_ERROR | The order_id parameter is missing |

### Static Wallet Errors

| HTTP Code | Error Code | Description |
|-----------|------------|-------------|
| 400 | INVALID_REQUEST | A merchant wallet already exists for this network/coin combination |
| 422 | VALIDATION_ERROR | The network parameter is missing |
| 422 | VALIDATION_ERROR | The coin parameter is missing |

### Withdrawal Errors

| HTTP Code | Error Code | Description |
|-----------|------------|-------------|
| 400 | INSUFFICIENT_BALANCE | The merchant doesn't have enough balance for the withdrawal |
| 403 | PERMISSION_DENIED | The API key doesn't have is_allowed_withdraw permission |
| 422 | VALIDATION_ERROR | The to_address parameter is missing |
| 422 | VALIDATION_ERROR | The amount parameter is missing |

## Best Practices for Error Handling

### Client-Side Error Handling

<div datatype="code-tabs">

```php
// PHP Example
try {
    // Make API request
    $response = $client->request('POST', 'https://api.coinssend.com/v1/invoices', [
        'headers' => [
            'Content-Type' => 'application/json',
            'Merchant' => $merchantId,
            'Sign' => $signature
        ],
        'json' => $requestData
    ]);
    
    $responseData = json_decode($response->getBody(), true);
    
    if ($responseData['status'] === 'success') {
        // Handle success
        return $responseData['data'];
    } else {
        // Handle API errors with status "error"
        throw new Exception($responseData['error']['message'], $responseData['error']['code']);
    }
} catch (ClientException $e) {
    // Handle HTTP errors (4xx)
    $responseBody = json_decode($e->getResponse()->getBody(), true);
    $errorMessage = $responseBody['error']['message'] ?? 'Unknown client error';
    $errorCode = $responseBody['error']['code'] ?? $e->getCode();
    
    // Log and handle the error
    error_log("API Error ($errorCode): $errorMessage");
    
    // Handle specific errors
    if ($errorCode === 401) {
        // Handle authentication errors
    } elseif ($errorCode === 422) {
        // Handle validation errors
        $validationErrors = $responseBody['error']['errors'] ?? [];
        // Process validation errors
    }
    
    throw new Exception($errorMessage, $errorCode);
} catch (ServerException $e) {
    // Handle server errors (5xx)
    error_log("Server Error: " . $e->getMessage());
    throw new Exception("The server encountered an error. Please try again later.", 500);
}
```

```javascript
// JavaScript Example
async function makeApiRequest(endpoint, data) {
  try {
    // Generate signature
    const signature = generateSignature(data, apiKey);
    
    const response = await fetch(`https://api.coinssend.com/v1/${endpoint}`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'Merchant': merchantId,
        'Sign': signature
      },
      body: JSON.stringify(data)
    });
    
    const responseData = await response.json();
    
    if (!response.ok) {
      // Handle HTTP error status
      const error = new Error(responseData.error?.message || 'API request failed');
      error.code = responseData.error?.code || response.status;
      error.validationErrors = responseData.error?.errors;
      throw error;
    }
    
    if (responseData.status === 'error') {
      // Handle application error with status "error"
      const error = new Error(responseData.error.message);
      error.code = responseData.error.code;
      error.validationErrors = responseData.error.errors;
      throw error;
    }
    
    return responseData.data;
  } catch (error) {
    // Handle network errors
    if (error.name === 'TypeError' && error.message.includes('Failed to fetch')) {
      console.error('Network error. Please check your connection.');
    }
    
    // Handle specific error codes
    if (error.code === 401) {
      console.error('Authentication failed. Please check your credentials.');
    } else if (error.code === 422 && error.validationErrors) {
      console.error('Validation errors:', error.validationErrors);
    } else if (error.code >= 500) {
      console.error('Server error. Please try again later.');
    }
    
    throw error;
  }
}
```

```python
# Python Example
import requests
import logging

def make_api_request(endpoint, data, merchant_id, api_key):
    try:
        # Generate signature
        signature = generate_signature(data, api_key)
        
        # Make API request
        response = requests.post(
            f'https://api.coinssend.com/v1/{endpoint}',
            headers={
                'Content-Type': 'application/json',
                'Merchant': merchant_id,
                'Sign': signature
            },
            json=data
        )
        
        # Parse response
        response_data = response.json()
        
        # Check HTTP status code
        response.raise_for_status()
        
        # Check API status
        if response_data.get('status') == 'error':
            error_data = response_data.get('error', {})
            error_message = error_data.get('message', 'Unknown API error')
            error_code = error_data.get('code', 0)
            validation_errors = error_data.get('errors')
            
            if validation_errors:
                logging.error(f"Validation errors: {validation_errors}")
            
            raise Exception(f"API Error ({error_code}): {error_message}")
        
        return response_data.get('data')
        
    except requests.exceptions.HTTPError as e:
        # Handle HTTP errors
        error_data = {}
        try:
            error_data = response.json().get('error', {})
        except:
            pass
        
        error_message = error_data.get('message', str(e))
        error_code = error_data.get('code', response.status_code)
        validation_errors = error_data.get('errors')
        
        # Handle specific errors
        if response.status_code == 401:
            logging.error("Authentication failed. Check credentials.")
        elif response.status_code == 422 and validation_errors:
            logging.error(f"Validation errors: {validation_errors}")
        elif response.status_code >= 500:
            logging.error("Server error. Please try again later.")
        
        raise Exception(f"API Error ({error_code}): {error_message}")
        
    except requests.exceptions.ConnectionError:
        # Handle connection errors
        logging.error("Connection error. Please check your network.")
        raise
```

</div>

### Retry Strategy

For certain types of errors, especially network errors and server errors (5xx), it's recommended to implement a retry strategy:

1. **Exponential Backoff**: Increase the delay between retries exponentially
2. **Jitter**: Add randomness to the retry delays to prevent thundering herd problems
3. **Maximum Retries**: Limit the number of retry attempts
4. **Retry Only Idempotent Operations**: Only retry GET, PUT, and DELETE requests automatically; be careful with POST requests

Example retry implementation:

<div datatype="code-tabs">

```php
// PHP Retry Example
function makeRequestWithRetry($client, $method, $url, $options, $maxRetries = 3) {
    $retries = 0;
    $shouldRetry = true;
    
    while ($shouldRetry && $retries <= $maxRetries) {
        try {
            if ($retries > 0) {
                // Add exponential backoff with jitter
                $delay = (2 ** $retries) * 100 + random_int(0, 100);
                usleep($delay * 1000); // Convert to microseconds
            }
            
            $response = $client->request($method, $url, $options);
            return json_decode($response->getBody(), true);
            
        } catch (Exception $e) {
            $retries++;
            $statusCode = $e->getCode();
            
            // Only retry on server errors and network errors
            $shouldRetry = (
                $retries <= $maxRetries && 
                ($statusCode >= 500 || $statusCode === 0)
            );
            
            if (!$shouldRetry) {
                throw $e;
            }
            
            error_log("Request failed, retrying ({$retries}/{$maxRetries}): " . $e->getMessage());
        }
    }
}
```

```javascript
// JavaScript Retry Example
async function makeRequestWithRetry(url, options, maxRetries = 3) {
  let retries = 0;
  
  while (retries <= maxRetries) {
    try {
      if (retries > 0) {
        // Add exponential backoff with jitter
        const delay = (2 ** retries) * 100 + Math.floor(Math.random() * 100);
        await new Promise(resolve => setTimeout(resolve, delay));
      }
      
      const response = await fetch(url, options);
      const data = await response.json();
      
      if (!response.ok && response.status >= 500 && retries < maxRetries) {
        // Server error, retry
        retries++;
        console.warn(`Request failed with ${response.status}, retrying (${retries}/${maxRetries})...`);
        continue;
      }
      
      return data;
      
    } catch (error) {
      // Network error
      if (retries < maxRetries) {
        retries++;
        console.warn(`Request failed with ${error.message}, retrying (${retries}/${maxRetries})...`);
      } else {
        throw error;
      }
    }
  }
}
```

```python
# Python Retry Example
import time
import random
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

def create_session_with_retry():
    session = requests.Session()
    
    # Configure retry strategy
    retry_strategy = Retry(
        total=3,  # Maximum number of retries
        backoff_factor=0.5,  # Exponential backoff factor
        status_forcelist=[500, 502, 503, 504],  # Retry on these status codes
        allowed_methods=["GET", "PUT", "DELETE", "HEAD", "OPTIONS"]  # Don't retry POST by default
    )
    
    # Apply retry strategy to the session
    adapter = HTTPAdapter(max_retries=retry_strategy)
    session.mount("https://", adapter)
    session.mount("http://", adapter)
    
    return session

# Usage
session = create_session_with_retry()
response = session.post("https://api.coinssend.com/v1/merchants/12345/wallet-address")
```

</div>

## Logging and Monitoring

Proper error handling should include:

1. **Logging**: Log all API errors with sufficient context for debugging
2. **Monitoring**: Monitor error rates to detect potential issues
3. **Alerting**: Set up alerts for critical errors or unusual error patterns

By implementing these best practices, you can build more resilient applications that handle errors gracefully and provide a better user experience.

## FAQ: Common Issues and Solutions

### Q: Why am I getting "Invalid sign" errors?

- Check that you're correctly generating the signature (see the [Authentication](./authentication.md) guide)
- Ensure the API key is correct and active
- Verify that the request body is identical between signature generation and the actual request
- Check for white space or encoding issues in the request body

### Q: How can I distinguish between a temporary server error and a permanent issue?

- HTTP 5xx status codes (500, 502, 503, 504) usually indicate temporary issues
- HTTP 4xx status codes (400, 401, 403, 404, 422) indicate client-side errors that won't be resolved by retrying

### Q: What should I do if I get rate limit errors (429)?

- Implement exponential backoff with jitter
- Reduce the frequency of your API requests
- Consider batching multiple operations into fewer requests when possible